@@ -9,10 +9,12 @@ import numpy as np
 import pandas as pd
 import shap
 from mpl_toolkits.axes_grid1 import ImageGrid
-from sklearn.metrics import (auc, average_precision_score, roc_auc_score,
-                             precision_recall_curve, roc_curve, mean_squared_error)
-from sklearn.preprocessing import label_binarize
 from scipy.special import softmax
+from sklearn.metrics import (auc, average_precision_score, mean_squared_error,
+                             precision_recall_curve, roc_auc_score, roc_curve)
+from sklearn.preprocessing import label_binarize
+
+import hipe4ml.tree_handler
 
 
 def _plot_output(df_train, df_test, lims, bins, label, color, kwds):
@@ -33,7 +35,8 @@ def _plot_output(df_train, df_test, lims, bins, label, color, kwds):
         hist = hist * scale
 
     center = (bins[:-1] + bins[1:]) / 2
-    plt.errorbar(center, hist, yerr=err, fmt='o', c=color, label=f'{label} pdf Test Set')
+    plt.errorbar(center, hist, yerr=err, fmt='o',
+                 c=color, label=f'{label} pdf Test Set')
 
 
 def plot_output_train_test(model, data, bins=80, output_margin=True, labels=None, logscale=False, **kwds):
@@ -41,7 +44,7 @@ def plot_output_train_test(model, data, bins=80, output_margin=True, labels=None
     Plot the model output distributions for each class and output
     both for training and test set.
 
-    Input
+    Parameters
     ----------------------------------------
     model: hipe4ml model handler
 
@@ -75,9 +78,9 @@ def plot_output_train_test(model, data, bins=80, output_margin=True, labels=None
     **kwds
         Extra arguments are passed on to plt.hist()
 
-    Output
+    Returns
     ----------------------------------------
-    res: list of matplotlib.figure.Figure
+    out: matplotlib.figure.Figure or list of them
         Model output distributions for each class
     """
     class_labels = np.unique(data[1])
@@ -86,29 +89,33 @@ def plot_output_train_test(model, data, bins=80, output_margin=True, labels=None
     prediction = []
     for xxx, yyy in ((data[0], data[1]), (data[2], data[3])):
         for class_lab in class_labels:
-            prediction.append(model.predict(xxx[yyy == class_lab], output_margin))
+            prediction.append(model.predict(
+                xxx[yyy == class_lab], output_margin))
 
     low = min(np.min(d) for d in prediction)
     high = max(np.max(d) for d in prediction)
     low_high = (low, high)
 
-    res = []
     # only one figure in case of binary classification
     if n_classes <= 2:
+        res = plt.figure()
         labels = ['Signal', 'Background'] if labels is None else labels
         colors = ['b', 'r']
-        res.append(plt.figure())
         for i_class, (label, color) in enumerate(zip(labels, colors)):
-            _plot_output(prediction[i_class], prediction[i_class+2], low_high, bins, label, color, kwds)
+            _plot_output(
+                prediction[i_class], prediction[i_class+2], low_high, bins, label, color, kwds)
         if logscale:
             plt.yscale('log')
         plt.xlabel('BDT output', fontsize=13, ha='right', position=(1, 20))
-        plt.ylabel('Counts (arb. units)', fontsize=13, horizontalalignment='left')
+        plt.ylabel('Counts (arb. units)', fontsize=13,
+                   horizontalalignment='left')
         plt.legend(frameon=False, fontsize=12, loc='best')
 
     # n figures in case of multi-classification with n classes
     else:
-        labels = [f'class{class_lab}' for class_lab in class_labels] if labels is None else labels
+        res = []
+        labels = [
+            f'class{class_lab}' for class_lab in class_labels] if labels is None else labels
         cmap = plt.cm.get_cmap('tab10')
         colors = [cmap(i_class) for i_class in range(len(labels))]
         for output, out_label in zip(class_labels, labels):
@@ -118,30 +125,28 @@ def plot_output_train_test(model, data, bins=80, output_margin=True, labels=None
                              label, color, kwds)
             if logscale:
                 plt.yscale('log')
-            plt.xlabel(f'BDT output for {out_label}', fontsize=13, ha='right', position=(1, 20))
-            plt.ylabel('Counts (arb. units)', fontsize=13, horizontalalignment='left')
+            plt.xlabel(f'BDT output for {out_label}',
+                       fontsize=13, ha='right', position=(1, 20))
+            plt.ylabel('Counts (arb. units)', fontsize=13,
+                       horizontalalignment='left')
             plt.legend(frameon=False, fontsize=12, loc='best')
 
     return res
 
-
-def plot_distr(list_of_df, column=None, figsize=None, bins=50, log=False, labels=None, alpha=0.5):
+# flake8: noqa: C901
+def plot_distr(data_list, column=None, bins=50, labels=None, colors=None, **kwds):  # pylint: disable=too-many-branches
     """
     Draw histograms comparing the distributions of each class.
 
-    Input
+    Parameters
     -----------------------------------------
-    list_of_df: list
-        Contains a dataframe for each class
+    data_list: TreeHandler, pandas.Dataframe or list of them
+        Contains a TreeHandler or a dataframe for each class
 
-    column: list
+    column: str or list of them
         Contains the name of the features you want to plot
         Example: ['dEdx', 'pT', 'ct']
 
-    figsize: list
-        The size in inches of the figure to create. Uses the value in
-        matplotlib.rcParams by default.
-
     bins: int or sequence of scalars or str
         If bins is an int, it defines the number of equal-width
         bins in the given range (10, by default). If bins is a
@@ -149,21 +154,32 @@ def plot_distr(list_of_df, column=None, figsize=None, bins=50, log=False, labels
         bin edges, including the rightmost edge, allowing for
         non-uniform bin widths.
 
-    log: bool
-        If True enable log scale plot
-
-    labels: list
+    labels: str or list of them
         Contains the labels to be displayed in the legend
         If None the labels are class1, class2, ..., classN
 
-    alpha: float
-        Value between 0 and 1, tune the histogram transparency
+    colors: str or list of them
+        List of the colors to be used to fill the histograms
 
-    Output
+    **kwds:
+        extra arguments are passed on to pandas.DataFrame.hist():
+        https://pandas.pydata.org/pandas-docs/stable/reference/api/pandas.DataFrame.hist.html
+
+    Returns
     -----------------------------------------
-    axes: numpy array of matplotlib.figure.Figure
+    out: numpy array of matplotlib.axes.AxesSubplot
         Distributions of the features for each class
     """
+    list_of_df = []
+    if not isinstance(data_list, list):
+        data_list = [data_list]
+        labels = [labels]
+        colors = [colors]
+    if isinstance(data_list[0], hipe4ml.tree_handler.TreeHandler):
+        for handl in data_list:
+            list_of_df.append(handl.get_data_frame())
+    else:
+        list_of_df = data_list
 
     if column is not None:
         if not isinstance(column, (list, np.ndarray, pd.Index)):
@@ -171,36 +187,36 @@ def plot_distr(list_of_df, column=None, figsize=None, bins=50, log=False, labels
         for dfm in list_of_df:
             dfm = dfm[column]
 
-    if figsize is None:
-        figsize = [20, 15]
-
     if labels is None:
         labels = [f'class{i_class}' for i_class, _ in enumerate(list_of_df)]
 
-    for i_class, (dfm, lab) in enumerate(zip(list_of_df, labels)):
+    if colors is None:
+        colors = [None for i_class, _ in enumerate(list_of_df)]
+
+    for i_class, (dfm, lab, col) in enumerate(zip(list_of_df, labels, colors)):
         if i_class == 0:
-            axes = dfm.hist(column=column, alpha=alpha, bins=bins, figsize=figsize, label=lab,
-                            density=True, grid=False, log=log)
+            axes = dfm.hist(column=column, bins=bins, label=lab, color=col, **kwds)
             axes = axes.flatten()
             axes = axes[:len(column)]
         else:
-            dfm.hist(ax=axes, column=column, alpha=alpha, bins=bins, figsize=figsize, label=lab,
-                     density=True, grid=False, log=log)
+            dfm.hist(ax=axes, column=column, bins=bins, label=lab, color=col, **kwds)
     for axs in axes:
-        axs.set_ylabel('Counts (arb. units)')
+        axs.set_ylabel('Counts')
     axes[-1].legend(loc='best')
+    if len(axes) == 1:
+        axes = axes[0]
     return axes
 
 
-def plot_corr(list_of_df, columns, labels=None, **kwds):
+def plot_corr(data_list, columns, labels=None, **kwds):
     """
     Calculate pairwise correlation between features for
     each class (e.g. signal and background in case of binary
     classification)
 
-    Input
+    Parameters
     -----------------------------------------------
-    list_of_df: list
+    data_list: list
         Contains dataframes for each class
 
     columns: list
@@ -213,12 +229,17 @@ def plot_corr(list_of_df, columns, labels=None, **kwds):
 
     **kwds: extra arguments are passed on to DataFrame.corr()
 
-    Output
+    Returns
     ------------------------------------------------
-    fig: list of matplotlib.figure.Figure
+    out: matplotlib.figure.Figure or list of them
         Correlations between the features for each class
     """
-
+    list_of_df = []
+    if isinstance(data_list[0], hipe4ml.tree_handler.TreeHandler):
+        for handl in data_list:
+            list_of_df.append(handl.get_data_frame())
+    else:
+        list_of_df = data_list
     corr_mat = []
     for dfm in list_of_df:
         dfm = dfm[columns]
@@ -226,18 +247,23 @@ def plot_corr(list_of_df, columns, labels=None, **kwds):
 
     if labels is None:
         labels = []
-        if len(corr_mat) > 2:
+        if len(corr_mat) != 2:
             for i_mat, _ in enumerate(corr_mat):
                 labels.append(f'class{i_mat}')
         else:
             labels.append('Signal')
             labels.append('Background')
 
-    fig = []
+    res = []
     for mat, lab in zip(corr_mat, labels):
-        fig.append(plt.figure(figsize=(8, 7)))
-        grid = ImageGrid(fig[-1], 111, axes_pad=0.15, nrows_ncols=(1, 1), share_all=True,
-                         cbar_location='right', cbar_mode='single', cbar_size='7%', cbar_pad=0.15)
+        if len(corr_mat) < 2:
+            res = plt.figure(figsize=(8, 7))
+            grid = ImageGrid(res, 111, axes_pad=0.15, nrows_ncols=(1, 1), share_all=True,
+                             cbar_location='right', cbar_mode='single', cbar_size='7%', cbar_pad=0.15)
+        else:
+            res.append(plt.figure(figsize=(8, 7)))
+            grid = ImageGrid(res[-1], 111, axes_pad=0.15, nrows_ncols=(1, 1), share_all=True,
+                             cbar_location='right', cbar_mode='single', cbar_size='7%', cbar_pad=0.15)
 
         opts = {'cmap': plt.get_cmap(
             'coolwarm'), 'vmin': -1, 'vmax': +1, 'snap': True}
@@ -263,7 +289,7 @@ def plot_corr(list_of_df, columns, labels=None, **kwds):
 
         axs.cax.colorbar(heatmap)
 
-    return fig
+    return res
 
 
 def plot_bdt_eff(threshold, eff_sig):
@@ -271,7 +297,7 @@ def plot_bdt_eff(threshold, eff_sig):
     Plot the model efficiency calculated with the function
     bdt_efficiency_array() in analysis_utils
 
-    Input
+    Parameters
     -----------------------------------
     threshold: array
         Score threshold array
@@ -279,9 +305,9 @@ def plot_bdt_eff(threshold, eff_sig):
     eff_sig: array
         model efficiency array
 
-    Output
+    Returns
     -----------------------------------
-    res: matplotlib.figure.Figure
+    out: matplotlib.figure.Figure
         Plot containing model efficiency as a
         function of the threshold score
     """
@@ -292,7 +318,6 @@ def plot_bdt_eff(threshold, eff_sig):
     plt.ylabel('Efficiency')
     plt.title('Efficiency vs Score')
     plt.grid()
-
     return res
 
 
@@ -307,9 +332,11 @@ def _plot_roc_ovr(y_truth, y_score, n_classes, labels, average):
     for i_class, lab in enumerate(labels):
         fpr, tpr, _ = roc_curve(y_truth_multi[:, i_class], y_score[:, i_class])
         roc_auc = auc(fpr, tpr)
-        plt.plot(fpr, tpr, lw=1, c=cmap(i_class), label=f'{lab} Vs Rest (AUC = {roc_auc:.4f})')
+        plt.plot(fpr, tpr, lw=1, c=cmap(i_class),
+                 label=f'{lab} Vs Rest (AUC = {roc_auc:.4f})')
     # compute global ROC AUC
-    global_roc_auc = roc_auc_score(y_truth, y_score, average=average, multi_class='ovr')
+    global_roc_auc = roc_auc_score(
+        y_truth, y_score, average=average, multi_class='ovr')
     plt.plot([], [], ' ', label=f'Average OvR ROC AUC: {global_roc_auc:.4f}')
 
 
@@ -334,7 +361,8 @@ def _plot_roc_ovo(y_truth, y_score, n_classes, labels, average):
         plt.plot(fpr_b, tpr_b, lw=1, c=cmap(i_comb), alpha=0.6,
                  label=f'{labels[bbb]} Vs {labels[aaa]} (AUC = {roc_auc_b:.4f})')
     # compute global ROC AUC
-    global_roc_auc = roc_auc_score(y_truth, y_score, average=average, multi_class='ovo')
+    global_roc_auc = roc_auc_score(
+        y_truth, y_score, average=average, multi_class='ovo')
     plt.plot([], [], ' ', label=f'Average OvO ROC AUC: {global_roc_auc:.4f}')
 
 
@@ -342,7 +370,7 @@ def plot_roc(y_truth, y_score, pos_label=None, labels=None, average='macro', mul
     """
     Calculate and plot the roc curve
 
-    Input
+    Parameters
     -------------------------------------
     y_truth: array
         True labels for the belonging class. If labels are not
@@ -374,9 +402,9 @@ def plot_roc(y_truth, y_score, pos_label=None, labels=None, average='macro', mul
         Option to compute ROC curves used only in case of multi-classification.
         The one-vs-one 'ovo' and one-vs-rest 'ovr' approaches are available
 
-    Output
+    Returns
     -------------------------------------
-    res: matplotlib.figure.Figure
+    out: matplotlib.figure.Figure
         Plot containing the roc curves
     """
     # get number of classes
@@ -430,7 +458,7 @@ def plot_roc_train_test(y_truth_test, y_score_test, y_truth_train, y_score_train
     """
     Calculate and plot the roc curve for test and train sets
 
-    Input
+    Parameters
     -------------------------------------
     y_truth_test: array
         True labels for the belonging class of the test set. If labels
@@ -473,14 +501,16 @@ def plot_roc_train_test(y_truth_test, y_score_test, y_truth_train, y_score_train
         Option to compute ROC curves used only in case of multi-classification.
         The one-vs-one 'ovo' and one-vs-rest 'ovr' approaches are available
 
-    Output
+    Returns
     -------------------------------------
-    res: matplotlib.figure.Figure
+    out: matplotlib.figure.Figure
         Plot containing the roc curves
     """
     # call plot_roc for both train and test sets
-    fig_test = plot_roc(y_truth_test, y_score_test, pos_label, labels, average, multi_class_opt)
-    fig_train = plot_roc(y_truth_train, y_score_train, pos_label, labels, average, multi_class_opt)
+    fig_test = plot_roc(y_truth_test, y_score_test,
+                        pos_label, labels, average, multi_class_opt)
+    fig_train = plot_roc(y_truth_train, y_score_train,
+                         pos_label, labels, average, multi_class_opt)
     axes_test = fig_test.get_axes()[0]
     axes_train = fig_train.get_axes()[0]
 
@@ -518,7 +548,7 @@ def plot_feature_imp(df_in, y_truth, model, labels=None, n_sample=10000, approxi
     each feature. The calculation is performed on a subsample of the
     input training/test set
 
-    Input
+    Parameters
     -------------------------------------------
     df_in: Pandas dataframe
         Training or test set dataframe
@@ -544,9 +574,9 @@ def plot_feature_imp(df_in, y_truth, model, labels=None, n_sample=10000, approxi
         Run fast and approximat roughly the SHAP values. For more information
         see https://shap.readthedocs.io/en/latest/#shap.TreeExplainer.shap_values
 
-    Output
+    Returns
     -------------------------------------------
-    res: list of matplotlib.figure.Figure
+    out: matplotlib.figure.Figure or list of them
         Plots with shap feature importance
     """
     class_labels, class_counts = np.unique(y_truth, return_counts=True)
@@ -563,16 +593,19 @@ def plot_feature_imp(df_in, y_truth, model, labels=None, n_sample=10000, approxi
     explainer = shap.TreeExplainer(model.get_original_model())
     shap_values = explainer.shap_values(df_subs, approximate=approximate)
 
-    res = []
     if n_classes <= 2:
-        res.append(plt.figure(figsize=(18, 9)))
-        shap.summary_plot(shap_values, df_subs, plot_size=(18, 9), class_names=labels, show=False)
+        res = plt.figure(figsize=(18, 9))
+        shap.summary_plot(shap_values, df_subs, plot_size=(
+            18, 9), class_names=labels, show=False)
     else:
+        res = []
         for i_class in range(n_classes):
             res.append(plt.figure(figsize=(18, 9)))
-            shap.summary_plot(shap_values[i_class], df_subs, plot_size=(18, 9), class_names=labels, show=False)
+            shap.summary_plot(shap_values[i_class], df_subs, plot_size=(
+                18, 9), class_names=labels, show=False)
         res.append(plt.figure(figsize=(18, 9)))
-        shap.summary_plot(shap_values, df_subs, plot_type='bar', plot_size=(18, 9), class_names=labels, show=False)
+        shap.summary_plot(shap_values, df_subs, plot_type='bar',
+                          plot_size=(18, 9), class_names=labels, show=False)
 
     return res
 
@@ -580,7 +613,7 @@ def plot_feature_imp(df_in, y_truth, model, labels=None, n_sample=10000, approxi
 def plot_precision_recall(y_truth, y_score, labels=None, pos_label=None):
     """ Plot precision recall curve
 
-    Input
+    Parameters
     -------------------------------------
     y_truth: array
         True labels for the belonging class. If labels are not
@@ -594,9 +627,9 @@ def plot_precision_recall(y_truth, y_score, labels=None, pos_label=None):
         if y_true is in {0, 1, ..., N}, pos_label is set to 1,
         otherwise an error will be raised.
 
-    Output
+    Returns
     -------------------------------------
-    res: matplotlib.figure.Figure
+    out: matplotlib.figure.Figure
         Plot containing the precision recall curves
     """
     # get number of classes
@@ -649,7 +682,7 @@ def plot_precision_recall(y_truth, y_score, labels=None, pos_label=None):
 def plot_learning_curves(model, data, n_points=10):
     """ Plot learning curves
 
-    Input
+    Parameters
     -------------------------------------
     model: hipe4ml model_handler
 
@@ -661,9 +694,9 @@ def plot_learning_curves(model, data, n_points=10):
     n_points: int
         Number of points used to sample the learning curves
 
-    Output
+    Returns
     -------------------------------------
-    res: matplotlib.figure.Figure
+    out: matplotlib.figure.Figure
         Plot containing the learning curves
     """
 
